using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;
using System.IO;
namespace Frets {
	public class FretsGame : Microsoft.Xna.Framework.Game {

		Interface.Menus Menus;
		

		GraphicsDeviceManager graphics;
		ContentManager content;

		private enum GameMode {
			InMenu,
			PlayingGame,
		}
		private GameMode CurrentMode = GameMode.InMenu;

		private FmodEx.Fmod SoundSystem;
		Game.Game CurrentGame;

		public FretsGame() {
			graphics = new GraphicsDeviceManager(this);
			content = new ContentManager(Services);
		}

		protected override void Initialize() {

			// Initialise sound system:
			this.SoundSystem = new FmodEx.Fmod();

			// Initialise game back-end:
			this.CurrentGame = new Game.Game(this.SoundSystem, @"C:\Program Files\Frets on Fire");

			// Set up menus:
			this.Menus = new Interface.Menus(this.CurrentGame);
			this.Menus.CurrentMenu = new Frets.Interface.Menus.MenuItem(null);
			
			Interface.Menus.MenuItem SongsMenu = new Frets.Interface.Menus.MenuItem(this.Menus.CurrentMenu, "Play Game");
			SongsMenu.Tag = "";
			SongsMenu.Entered += new EventHandler(SongsMenu_Entered);
				this.Menus.CurrentMenu.ChildMenus.Add(SongsMenu);
			
			Interface.Menus.MenuItem OptionsMenu = new Frets.Interface.Menus.MenuItem(this.Menus.CurrentMenu, "Options");
			this.Menus.CurrentMenu.ChildMenus.Add(OptionsMenu);
			OptionsMenu.ChildMenus.Add(new Frets.Interface.Menus.MenuItem(OptionsMenu, "Game Options"));
			OptionsMenu.ChildMenus.Add(new Frets.Interface.Menus.MenuItem(OptionsMenu, "Video Options"));
			OptionsMenu.ChildMenus.Add(new Frets.Interface.Menus.MenuItem(OptionsMenu, "Sound Options"));

			

			this.Menus.CurrentMenu.ChildMenus.Add(new Frets.Interface.Menus.MenuItem(this.Menus.CurrentMenu, "Quit"));
			this.Menus.SelectedMenu = this.Menus.CurrentMenu.ChildMenus[0];
			base.Initialize();
		}

		void SongsMenu_Entered(object sender, EventArgs e) {
			Interface.Menus.MenuItem MI = sender as Interface.Menus.MenuItem;
			MI.ChildMenus.Clear();
			foreach (string s in this.CurrentGame.GetDirectories((sender as Interface.Menus.MenuItem).Tag as string)) {
				Interface.Menus.MenuItem ChangeDirectory = new Frets.Interface.Menus.MenuItem(MI, Path.GetFileName(s), s);
				MI.ChildMenus.Add(ChangeDirectory);
				ChangeDirectory.Entered += new EventHandler(SongsMenu_Entered);
			}
			foreach (Game.Song LoadingSong in this.CurrentGame.GetSongs((sender as Interface.Menus.MenuItem).Tag as string)) {
				Interface.Menus.MenuItem PlaySong = new Frets.Interface.Menus.MenuItem(MI, LoadingSong.Name, LoadingSong);
				MI.ChildMenus.Add(PlaySong);
				PlaySong.Entered += new EventHandler(SelectSongDifficulty);
			}
		}

		void SelectSongDifficulty(object sender, EventArgs e) {
			Interface.Menus.MenuItem MI = sender as Interface.Menus.MenuItem;
			
			Game.Song SongToPlay = MI.Tag as Game.Song;
			SongToPlay.LoadNotes();

			MI.ChildMenus.Clear();

			Game.Song.Notes.DifficultyModes Difficulties = SongToPlay.AvailableDifficulties;

			Game.Song.Notes.DifficultyModes[] AllDifficulties = new Frets.Game.Song.Notes.DifficultyModes[]{
				Game.Song.Notes.DifficultyModes.Amazing, 
				Game.Song.Notes.DifficultyModes.Medium, 
				Game.Song.Notes.DifficultyModes.Easy, 
				Game.Song.Notes.DifficultyModes.Supaeasy, 
			};

			for (int i = 0; i < AllDifficulties.Length; ++i) {
				Game.Song.Notes.DifficultyModes DifficultyOption = AllDifficulties[i];

				if ((Difficulties & DifficultyOption) != Game.Song.Notes.DifficultyModes.None) {
					Interface.Menus.MenuItem DifficultyMenu = new Frets.Interface.Menus.MenuItem(MI, AllDifficulties[i].ToString());
					MI.ChildMenus.Add(DifficultyMenu);
					
					// Naughty :(
					DifficultyMenu.Entered += delegate(object sender2, EventArgs e2) {
						this.CurrentGame.LoadSong(SongToPlay);
						this.CurrentGame.StartSong(DifficultyOption);
						this.CurrentMode = GameMode.PlayingGame;
					};
				}
			}

			
		}

		/*
		void StartPlaySong(object sender, EventArgs e) {
			Interface.Menus.MenuItem MI = sender as Interface.Menus.MenuItem;
			Game.Song SongToPlay = MI.Tag as Game.Song;
			this.CurrentGame.LoadSong(SongToPlay);
			this.CurrentGame.StartSong(Frets.Game.Song.Notes.DifficultyModes.Amazing);
			this.CurrentMode = GameMode.PlayingGame;
		}
		*/
		protected override void LoadGraphicsContent(bool loadAllContent) {
			if (loadAllContent) {
				
			}
			this.Menus.LoadContent(content, graphics);
			this.CurrentGame.LoadContent(content, graphics);
		}

		


		protected override void UnloadGraphicsContent(bool unloadAllContent) {
			if (unloadAllContent == true) {
				content.Unload();
			}
		}


		private KeyboardState LastUpdateKeyboardState;
		protected override void Update(GameTime gameTime) {
			// Allows the default game to exit on Xbox 360 and Windows.
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			KeyboardState ThisUpdateKeyboardState = Keyboard.GetState();

			switch (this.CurrentMode) {
				case GameMode.InMenu:
					if (ThisUpdateKeyboardState.IsKeyDown(Keys.Down) && LastUpdateKeyboardState.IsKeyUp(Keys.Down)) this.Menus.MoveDown();
					if (ThisUpdateKeyboardState.IsKeyDown(Keys.Up) && LastUpdateKeyboardState.IsKeyUp(Keys.Up)) this.Menus.MoveUp();
					if (ThisUpdateKeyboardState.IsKeyDown(Keys.Enter) && LastUpdateKeyboardState.IsKeyUp(Keys.Enter)) this.Menus.Enter();
					if (ThisUpdateKeyboardState.IsKeyDown(Keys.Escape) && LastUpdateKeyboardState.IsKeyUp(Keys.Escape)) this.Menus.GoBack();
					break;
				case GameMode.PlayingGame:
					this.CurrentGame.Fret0Pressed = ThisUpdateKeyboardState.IsKeyDown(Keys.F5);
					this.CurrentGame.Fret1Pressed = ThisUpdateKeyboardState.IsKeyDown(Keys.F6);
					this.CurrentGame.Fret2Pressed = ThisUpdateKeyboardState.IsKeyDown(Keys.F7);
					this.CurrentGame.Fret3Pressed = ThisUpdateKeyboardState.IsKeyDown(Keys.F8);
					this.CurrentGame.Fret4Pressed = ThisUpdateKeyboardState.IsKeyDown(Keys.F9);
					this.CurrentGame.Pick0Pressed = ThisUpdateKeyboardState.IsKeyDown(Keys.Enter);
					this.CurrentGame.Update();
					break;
			}


			LastUpdateKeyboardState = ThisUpdateKeyboardState;
			base.Update(gameTime);
		}


		protected override void Draw(GameTime gameTime) {

			graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

			switch (this.CurrentMode) {
				case GameMode.InMenu:
					this.Menus.Draw(graphics);
					break;
				case GameMode.PlayingGame:
					CurrentGame.Draw(graphics);
					break;
			}
			

			base.Draw(gameTime);

		}



	}
}
